import os

# 是否开发模式。
def is_debug_mode():
    '''
    一般Python项目，通过`Run/Debug Configurations`配置，
    可设置启动参数`Parameters`，如`--debug`开启开发模式，通过命令行运行时也可以添加`--debug`来开启
    但Django的`Run/Debug Configurations`中
    只能修改环境变量，添加`DEBUG=1`开启
    而通过命令行直接运行，没有设置`DEBUG`环境变量，所以值会是`False`
    '''
    # 检查命令行参数`--debug`是否存在
    for arg in os.sys.argv:
        if arg == '--debug':
            return True

    # 检查环境变量`DEBUG`是否为`1`
    if os.environ.get("DEBUG") == '1':
        return True
    return False


def is_local_mode():
    '''
    判断是否本地模式。是因为本地连接的数据库和线上的不一样，诸如类似区别，所以需要判断
    参考`is_debug_mode`
    '''
    for arg in os.sys.argv:
        if arg == '--local':
            return True
    if os.environ.get("LOCAL") == '1':
        return True
    return False


def dev_dis(dev, dis):
    '''
    获取和开发模式相关的变量
    如果是开发模式，返回开发环境下的配置
    否则返回生产环境下的配置
    '''
    return dev if is_debug_mode() else dis

def loc_srv(loc, srv):
    '''
    参考`dev_dis`
    '''
    return loc if is_local_mode() else srv

if __name__ == '__main__':
    print(is_debug_mode())
    print(is_local_mode())
    print(dev_dis('dev', 'dis'))
    print(loc_srv('loc', 'srv'))



'''
异步执行
'''
import asyncio


async def make_async(func, it, i, count, semaphore):
    '''
    会把同步方法如`requests`异步化
    '''
    async with semaphore:
        await asyncio.to_thread(func, it, i, count)


async def make_tasks(task_func, task_data, sem):
    '''
    把需要处理的数据，循环提供给处理方法
    并用信号量进行限制
    '''
    semaphore = asyncio.Semaphore(sem)
    tasks = []
    count = len(task_data)
    for i in range(count):
        it = task_data[i]
        task = make_async(task_func, it, i, count, semaphore)
        tasks.append(task)
    await asyncio.gather(*tasks)


def async_call(task_func, task_data, sem):
    '''
    task_func: 数据处理的方法
        `func(item, index, total)`
        item: 每个要处理的数据
        index: 当前索引 - 便于掌握进度
        total: 数据总数
    task_data: 需要处理的数据列表
    sem: 同时最多处理的数量
    '''
    asyncio.run(make_tasks(task_func, task_data, sem))


if __name__ == '__main__':
    import requests
    result = []
    def go(it, index, total):
        print(index)
        requests.get('http://www.baidu.com')
        result.append([it, index, total])

    async_call(go, range(10), 3)
    for p in result:
        print(p)




'''
输出方法执行的日志以及消耗时间
'''
import time, functools, inspect


default_printer = print # 全局默认的输出方法
default_starter = '开始'
default_starter_ch = '>>>'
default_ender = '结束'
default_ender_ch = '<<<'
default_cost_ch = '耗时'
default_time_ch = '秒'
default_stack_level = 3


def func_time(func_desc=None, printer=None):
    def decorator(func):
        desc = func_desc if func_desc else func.__name__
        prtr = printer if printer else default_printer # 如果不提供输出方法，使用默认的输出方法
        prtr = prtr if prtr else print # 如果默认的输出方法也没有，使用print方法
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            kw = {}
            try:
                if bool(inspect.signature(prtr).parameters.get('kwargs')) or bool(inspect.signature(prtr).parameters.get('stacklevel')):
                    kw['stacklevel'] = default_stack_level # 针对`alog.Logger`参数，适配调用堆栈层级，输出方法真正调用点的信息
            except:
                pass
            prtr(f'{desc}{default_starter} {default_starter_ch}', **kw)
            __start__ = time.time()
            result = func(*args, **kwargs)
            __end__ = time.time()
            prtr(f'{desc}{default_ender} {default_ender_ch} {default_cost_ch}: {__end__ - __start__:.1f}{default_time_ch}', **kw)
            return result
        return wrapper
    return decorator


def test():
    @func_time('某个方法', print)
    def some_func(n):
        time.sleep(n)

    some_func(1)


if __name__ == '__main__':
    test()
    pass



import concurrent.futures
from typing import Callable, Optional

EachCallback = Callable[[int, any, Optional[Exception]], None] # 每个任务回调方法，参数1：数据索引，参数2：返回值，参数3：异常

class Concurrent(object):
    def __init__(self, action, data_list, max_workers=3, each_callback:EachCallback=None):
        '''
        :param action: 每个线程执行的函数
        :param data_list: 数据列表
        :param max_workers: 最大线程数
        :param each_callback: 每个任务回调方法
        '''
        self.action = action
        self.data_list = data_list
        self.max_workers = max_workers
        self.each_callback = each_callback


    def start(self):
        ''' 开始执行 '''
        total = len(self.data_list)
        # 创建线程池
        with concurrent.futures.ThreadPoolExecutor(max_workers=self.max_workers) as executor:
            futures = {executor.submit(self.action, i, data, total):i for i, data in enumerate(self.data_list)}
            if callable(self.each_callback):
                for future in concurrent.futures.as_completed(futures):
                    index = futures[future]
                    try:
                        result = future.result()
                        self.each_callback(index, result, None)
                    except Exception as error:
                        self.each_callback(index, None, error)
    pass
